
import os
import sys
import json

import openpyxl
    
## Fix up names to match those used in DBpedia ESC

# In Excel data:
#   The Netherlands (also in one case 'The Netherands')
#   Serbia & Montenegro
#   Bosnia & Herzegovina
#   F.Y.R. Macedonia (2012)


# From ESC:
#   Netherlands (not The Netherlands)
#   Serbia and Montenegro
#   Bosnia and Herzegovina
#   Macedonia

#*****
# TODO:
# Some values noted in ESC that potentially need to be changed when queries via SPARQL query point
#   Republic of Macedonia (2005)
#   Federal Republic of Yugoslavia (1992)


excel_to_esc_lod_errata = {
    "The Netherlands":      "Netherlands",
    "The Netherands":       "Netherlands",
    "Serbia & Montenegro":  "Serbia and Montenegro",
    "Bosnia & Herzegovina": "Bosnia and Herzegovina",
    "F.Y.R. Macedonia":     "Macedonia"
}

def filter_finalist_votes(entry):
    return entry.get("(semi-) final") == "f" and entry.get('Duplicate') == None

def filter_jury_votes(entry):
    return entry.get("Jury or Televoting") == "J" and entry.get('Duplicate') == None

def filter_tele_votes(entry):
    return entry.get("Jury or Televoting") == "T" and entry.get('Duplicate') == None


def sortkey_year_to_country(entry):
    return str(entry.get('Year'))+entry.get('To country')

def sortkey_year_from_country(entry):
    return str(entry.get('Year'))+entry.get('From country')



# 1997:
# Televoting by
#  Austria, Germany, Sweeden, Switzerland, United Kingdom

# 1998-2000: Televote, with Jury voting as backup plan

# 2001-2002: All televote, or 50-50 mix if country so chooses, Jury backup
#    2001: Jury: Bosnia & Herzegovina, Turkey, Russia
#    2001: Mix:  Croatia, Greece, Malta
#
#    2002: Jury: Bosnia & Herzegovina, Turkey, Russia, FYR Macedonia, Romania
#    2002: Mix:  Cyprus, Greece, Spain, Croatia, Finland, Malta, Slovenia, Lithuania

# 2003: back to all Televote, with Jury voting as backup
# 2004-2008: same as 2003

# 2009-2012: Combined, except San Marino which was 100% Jury (due to size), Jury as backup

# 2013-2015: As before, but how two types of votes combined different

# 2016-      Jury and Tele given separately, added in

def voting_mode(year):

    vote_mode = None

    if (year>=2013):
        # Separate voting: Jury and Tele 
        vote_mode = "Separated"
    elif ((year>=2001) and (year<=2012)):
        vote_mode = "Combined"
    else:
        vote_mode = "Single"

    return vote_mode


def nuanced_voting_type(country,year):

    vote_type = None

    if (year>=2013):
        # Separate voting: Jury and Tele 
        vote_type = "JT"
        
    elif ((year>=2009) and (year<=2012)):
        if (country=="San Marino"):
            vote_type = "J"
        else:
            # Combined Jury and Tele
            vote_type = "C"
    elif ((year>=2003) and (year<=2008)):
        vote_type = "T"
    elif (year==2002):
        jury_list = [ "Bosnia and Herzegovina", "Turkey", "Russia", "FYR Macedonia", "Romania"]
        comb_list = [ "Cyprus", "Greece", "Spain", "Croatia", "Finland", "Malta", "Slovenia", "Lithuania" ]
        if (country in jury_list):
            vote_type = "J"
        elif (country in comb_list):
            vote_type = "C"
        else:
            vote_type = "T"
    elif (year==2001):
        jury_list = [ "Bosnia and Herzegovina", "Turkey", "Russia" ]
        comb_list = [ "Croatia", "Greece", "Malta" ]
        if (country in jury_list):
            vote_type = "J"
        elif (country in comb_list):
            vote_type = "C"
        else:
            vote_type = "T"
    elif ((year>=1998) and (year<=2000)):
        vote_type = "T"
    elif (year==1997):
        tele_list = [ "Austria", "Germany", "Sweeden", "Switzerland", "United Kingdom" ]
        if (country in tele_list):
            vote_type = "T"
        else:
            vote_type = "J"
    else:
        vote_type = "J"

    return vote_type



def load_xslx_sheet(excel_input_filename,sheetname):
    print "Loading: " + excel_input_filename
    print "  ..."
    workbook = openpyxl.load_workbook(excel_input_filename, data_only=True)
    print "  ......Done"

    print "  Loaded worksheets: ",  workbook.sheetnames

    worksheet = None
    
    if (sheetname == None):
        print "  Selecting default/active worksheet"
        worksheet = workbook.active
    else:
        print "  Selecting worksheet '"+sheetname+"'"
        worksheet = workbook[sheetname]

    print ""
    return worksheet


def header_to_column_mapping(worksheet,header_row_pos):

    # For header line, build up hashmap of header-names to column number

    header_names_mapping = {}

    min_col = worksheet.min_column
    max_col = worksheet.max_column

    found_header = False
    for col_j in range(min_col,max_col+1):
        header_cell = worksheet.cell(row=header_row_pos, column=col_j)
        header_cell_value = header_cell.value.strip()

            
        if header_cell_value:
            header_names_mapping[header_cell_value] = col_j

    return header_names_mapping


def convert_worksheet_to_hashmaps(worksheet):

    print "Converting worksheet to array header-name keyed array of hashmaps"
    
    data_hashmap_array = [];

    # scan through rows until non-empty/headernames encountered

    header_row_pos = None;
    
    row_i = 1
    for row in worksheet.values:
        for value in row:
            if (value != None) :
                header_row_pos = row_i
                break
        
            print(value)
        if (header_row_pos != None): break
        row_i = row_i + 1
     
    print "  Found headername data in row: ", header_row_pos
        
    header_names_mapping = header_to_column_mapping(worksheet,header_row_pos)
    header_names = header_names_mapping.keys()

    # Move on to next row, to start processing the data values
    row_i = row_i + 1
    num_rows = worksheet.max_row;
    
    while row_i<=num_rows:
        # Work through the provided header names, building up hashmap of header-name to value
        data_hashmap = {}
        
        for header_name in header_names:

            header_col_j = header_names_mapping[header_name]

            data_cell = worksheet.cell(row=row_i,column=header_col_j)
            data_value = data_cell.value


            if ((header_name == "To country") or (header_name == "From country")):
                if (data_value in excel_to_esc_lod_errata):
                    data_value = excel_to_esc_lod_errata.get(data_value)
                
            data_hashmap[header_name] = data_value

        row_i = row_i + 1
        
        data_hashmap_array.append(data_hashmap)

    print ""
    return data_hashmap_array

def save_greenstone_json_metadata(greenstone_metadata_json,json_output_filename):

    with open(json_output_filename, 'w') as outfile:
        json.dump(greenstone_metadata_json, outfile, indent=2)
